@@ -909,6 +909,12 @@ int MDI_Check_node_exists(const char* node_name, MDI_Comm comm, int* flag)
     return 1;
   }
 
+  // Only rank 0 should respond to this call
+  code* this_code = get_code(current_code);
+  if ( this_code->intra_rank != 0 ) {
+    return 0;
+  }
+
   // confirm that the node_name size is not greater than MDI_COMMAND_LENGTH
   if ( strlen(node_name) > COMMAND_LENGTH ) {
     mdi_error("Node name is greater than MDI_COMMAND_LENGTH");
@@ -961,6 +967,12 @@ int MDI_Get_nnodes(MDI_Comm comm, int* nnodes)
     return 1;
   }
 
+  // Only rank 0 should respond to this call
+  code* this_code = get_code(current_code);
+  if ( this_code->intra_rank != 0 ) {
+    return 0;
+  }
+
   vector* node_vec = get_node_vector(comm);
   *nnodes = (int)node_vec->size;
 
@@ -1004,6 +1016,13 @@ int MDI_Get_node(int index, MDI_Comm comm, char* name)
     mdi_error("MDI_Get_Node called but MDI has not been initialized");
     return 1;
   }
+
+  // Only rank 0 should respond to this call
+  code* this_code = get_code(current_code);
+  if ( this_code->intra_rank != 0 ) {
+    return 0;
+  }
+
   vector* node_vec = get_node_vector(comm);
   if ( node_vec == NULL ) {
     mdi_error("MDI_Get_Node unable to find node vector");
@@ -1096,6 +1115,12 @@ int MDI_Check_command_exists(const char* node_name, const char* command_name, MD
     return 1;
   }
 
+  // Only rank 0 should respond to this call
+  code* this_code = get_code(current_code);
+  if ( this_code->intra_rank != 0 ) {
+    return 0;
+  }
+
   // confirm that the node_name size is not greater than MDI_COMMAND_LENGTH
   if ( strlen(node_name) > COMMAND_LENGTH ) {
     mdi_error("Node name is greater than MDI_COMMAND_LENGTH");
@@ -1169,6 +1194,12 @@ int MDI_Get_ncommands(const char* node_name, MDI_Comm comm, int* ncommands)
     return 1;
   }
 
+  // Only rank 0 should respond to this call
+  code* this_code = get_code(current_code);
+  if ( this_code->intra_rank != 0 ) {
+    return 0;
+  }
+
   // confirm that the node_name size is not greater than MDI_COMMAND_LENGTH
   if ( strlen(node_name) > COMMAND_LENGTH ) {
     mdi_error("Node name is greater than MDI_COMMAND_LENGTH");
@@ -1230,6 +1261,13 @@ int MDI_Get_command(const char* node_name, int index, MDI_Comm comm, char* name)
     mdi_error("MDI_Get_Command called but MDI has not been initialized");
     return 1;
   }
+
+  // Only rank 0 should respond to this call
+  code* this_code = get_code(current_code);
+  if ( this_code->intra_rank != 0 ) {
+    return 0;
+  }
+
   vector* node_vec = get_node_vector(comm);
 
   // find the node
@@ -1327,6 +1365,12 @@ int MDI_Check_callback_exists(const char* node_name, const char* callback_name,
     return 1;
   }
 
+  // Only rank 0 should respond to this call
+  code* this_code = get_code(current_code);
+  if ( this_code->intra_rank != 0 ) {
+    return 0;
+  }
+
   // confirm that the node_name size is not greater than MDI_COMMAND_LENGTH
   if ( strlen(node_name) > COMMAND_LENGTH ) {
     mdi_error("Node name is greater than MDI_COMMAND_LENGTH");
@@ -1400,6 +1444,12 @@ int MDI_Get_ncallbacks(const char* node_name, MDI_Comm comm, int* ncallbacks)
     return 1;
   }
 
+  // Only rank 0 should respond to this call
+  code* this_code = get_code(current_code);
+  if ( this_code->intra_rank != 0 ) {
+    return 0;
+  }
+
   // confirm that the node_name size is not greater than MDI_COMMAND_LENGTH
   if ( strlen(node_name) > COMMAND_LENGTH ) {
     mdi_error("Node name is greater than MDI_COMMAND_LENGTH");
@@ -1462,6 +1512,12 @@ int MDI_Get_callback(const char* node_name, int index, MDI_Comm comm, char* name
     return 1;
   }
 
+  // Only rank 0 should respond to this call
+  code* this_code = get_code(current_code);
+  if ( this_code->intra_rank != 0 ) {
+    return 0;
+  }
+
   vector* node_vec = get_node_vector(comm);
 
   // find the node
@@ -1623,6 +1679,66 @@ int MDI_Get_plugin_mode(int* plugin_mode_ptr) {
 }
 
 
+/*! \brief Get plugin_argc
+ *
+ */
+int MDI_Plugin_get_argc(int* argc_ptr) {
+  if ( ! plugin_mode ) {
+    mdi_error("MDI_Plugin_get_argc called, but plugin mode is not active.");
+    return 1;
+  }
+  *argc_ptr = plugin_argc;
+  return 0;
+}
+
+
+/*! \brief Get plugin_argv
+ *
+ */
+int MDI_Plugin_get_argv(char*** argv_ptr) {
+  if ( ! plugin_mode ) {
+    mdi_error("MDI_Plugin_get_argv called, but plugin mode is not active.");
+    return 1;
+  }
+  *argv_ptr = plugin_argv;
+  return 0;
+}
+
+
+/*! \brief Get plugin_unedited_options
+ *
+ */
+int MDI_Plugin_get_args(char** args_ptr) {
+  if ( ! plugin_mode ) {
+    mdi_error("MDI_Plugin_get_args called, but plugin mode is not active.");
+    return 1;
+  }
+  *args_ptr = plugin_unedited_options;
+  return 0;
+}
+
+
+/*! \brief Get a specific element from plugin_argv
+ *
+ */
+int MDI_Plugin_get_arg(int index, char** arg_ptr) {
+  if ( ! plugin_mode ) {
+    mdi_error("MDI_Plugin_get_arg called, but plugin mode is not active.");
+    return 1;
+  }
+  if ( index < 0 ) {
+    mdi_error("MDI_Plugin_get_arg called with invalid value (<0) for index.");
+    return 1;
+  }
+  if ( index > plugin_argc ) {
+    mdi_error("MDI_Plugin_get_arg called with invalid value (>argc) for index.");
+    return 1;
+  }
+  *arg_ptr = plugin_argv[index];
+  return 0;
+}
+
+
 /*! \brief Get the Python plugin MPI communicator
  *
  */
