@@ -30,18 +30,15 @@ mdi_path = os.path.join( dir_path, mdi_name )
 try:
     mdi = ctypes.CDLL(mdi_path)
     MDI_COMMAND_LENGTH = ctypes.c_int.in_dll(mdi, "MDI_COMMAND_LENGTH").value
-except (ValueError, AttributeError):
+except (ValueError, AttributeError, OSError) as e:
     mdi = ctypes.WinDLL(mdi_path)
     MDI_COMMAND_LENGTH = ctypes.c_int.in_dll(mdi, "MDI_COMMAND_LENGTH").value
 
 # MDI Variables
 MDI_COMMAND_LENGTH = ctypes.c_int.in_dll(mdi, "MDI_COMMAND_LENGTH").value
 MDI_NAME_LENGTH = ctypes.c_int.in_dll(mdi, "MDI_NAME_LENGTH").value
+MDI_LABEL_LENGTH = ctypes.c_int.in_dll(mdi, "MDI_LABEL_LENGTH").value
 MDI_COMM_NULL = ctypes.c_int.in_dll(mdi, "MDI_COMM_NULL").value
-MDI_INT = ctypes.c_int.in_dll(mdi, "MDI_INT").value
-MDI_DOUBLE = ctypes.c_int.in_dll(mdi, "MDI_DOUBLE").value
-MDI_CHAR = ctypes.c_int.in_dll(mdi, "MDI_CHAR").value
-MDI_BYTE = ctypes.c_int.in_dll(mdi, "MDI_BYTE").value
 MDI_TCP = ctypes.c_int.in_dll(mdi, "MDI_TCP").value
 MDI_MPI = ctypes.c_int.in_dll(mdi, "MDI_MPI").value
 MDI_LINK = ctypes.c_int.in_dll(mdi, "MDI_LINK").value
@@ -52,6 +49,21 @@ MDI_MAJOR_VERSION = ctypes.c_int.in_dll(mdi, "MDI_MAJOR_VERSION").value
 MDI_MINOR_VERSION = ctypes.c_int.in_dll(mdi, "MDI_MINOR_VERSION").value
 MDI_PATCH_VERSION = ctypes.c_int.in_dll(mdi, "MDI_PATCH_VERSION").value
 
+# MDI Datatypes
+MDI_INT = ctypes.c_int.in_dll(mdi, "MDI_INT").value
+MDI_INT8_T = ctypes.c_int.in_dll(mdi, "MDI_INT8_T").value
+MDI_INT16_T = ctypes.c_int.in_dll(mdi, "MDI_INT16_T").value
+MDI_INT32_T = ctypes.c_int.in_dll(mdi, "MDI_INT32_T").value
+MDI_INT64_T = ctypes.c_int.in_dll(mdi, "MDI_INT64_T").value
+MDI_UINT8_T = ctypes.c_int.in_dll(mdi, "MDI_UINT8_T").value
+MDI_UINT16_T = ctypes.c_int.in_dll(mdi, "MDI_UINT16_T").value
+MDI_UINT32_T = ctypes.c_int.in_dll(mdi, "MDI_UINT32_T").value
+MDI_UINT64_T = ctypes.c_int.in_dll(mdi, "MDI_UINT64_T").value
+MDI_DOUBLE = ctypes.c_int.in_dll(mdi, "MDI_DOUBLE").value
+MDI_CHAR = ctypes.c_int.in_dll(mdi, "MDI_CHAR").value
+MDI_FLOAT = ctypes.c_int.in_dll(mdi, "MDI_FLOAT").value
+MDI_BYTE = ctypes.c_int.in_dll(mdi, "MDI_BYTE").value
+
 world_comm = None
 
 intra_code_comm = None
@@ -113,12 +125,39 @@ def mpi4py_get_np_array(buf, count, datatype, mdi_comm):
     if datatype == MDI_INT:
         mpi_type = MPI.INT
         datasize = ctypes.sizeof( ctypes.c_int )
+    elif datatype == MDI_INT8_T:
+        mpi_type = MPI.INT8_T
+        datasize = ctypes.sizeof( ctypes.c_int8 )
+    elif datatype == MDI_INT16_T:
+        mpi_type = MPI.INT16_T
+        datasize = ctypes.sizeof( ctypes.c_int16 )
+    elif datatype == MDI_INT32_T:
+        mpi_type = MPI.INT32_T
+        datasize = ctypes.sizeof( ctypes.c_int32 )
+    elif datatype == MDI_INT64_T:
+        mpi_type = MPI.INT64_T
+        datasize = ctypes.sizeof( ctypes.c_int64 )
+    elif datatype == MDI_UINT8_T:
+        mpi_type = MPI.UINT8_T
+        datasize = ctypes.sizeof( ctypes.c_uint8 )
+    elif datatype == MDI_UINT16_T:
+        mpi_type = MPI.UINT16_T
+        datasize = ctypes.sizeof( ctypes.c_uint16 )
+    elif datatype == MDI_UINT32_T:
+        mpi_type = MPI.UINT32_T
+        datasize = ctypes.sizeof( ctypes.c_uint32 )
+    elif datatype == MDI_UINT64_T:
+        mpi_type = MPI.UINT64_T
+        datasize = ctypes.sizeof( ctypes.c_uint64 )
     elif datatype == MDI_DOUBLE:
         mpi_type = MPI.DOUBLE
         datasize = ctypes.sizeof( ctypes.c_double )
     elif datatype == MDI_CHAR:
         mpi_type = MPI.CHAR
         datasize = ctypes.sizeof( ctypes.c_char )
+    elif datatype == MDI_FLOAT:
+        mpi_type = MPI.FLOAT
+        datasize = ctypes.sizeof( ctypes.c_float )
     elif datatype == MDI_BYTE:
         mpi_type = MPI.BYTE
         datasize = ctypes.sizeof( ctypes.c_char )
@@ -402,14 +441,71 @@ def set_mpi4py_split_callback():
     mdi.MDI_Set_Mpi4py_Split_Callback( mpi4py_split_callback_c )
 
 
+# MDI_Get_plugin_mode
+mdi.MDI_Get_plugin_mode.argtypes = [ctypes.POINTER(ctypes.c_int)]
+mdi.MDI_Get_plugin_mode.restype = ctypes.c_int
+def MDI_Get_plugin_mode():
+    plugin_mode = ctypes.c_int()
+    ret = mdi.MDI_Get_plugin_mode(ctypes.byref(plugin_mode))
+    if ret != 0:
+        raise Exception("MDI Error: MDI_Get_plugin_mode failed")
+    return plugin_mode.value
+
+
+# MDI_Get_plugin_mode
+mdi.MDI_Get_python_plugin_mpi_world_ptr.argtypes = [ctypes.POINTER(ctypes.c_void_p)]
+mdi.MDI_Get_python_plugin_mpi_world_ptr.restype = ctypes.c_int
+def MDI_Get_python_plugin_mpi_world_ptr():
+    python_plugin_mpi_world_ptr = ctypes.c_void_p()
+    ret = mdi.MDI_Get_python_plugin_mpi_world_ptr(ctypes.byref(python_plugin_mpi_world_ptr))
+    if ret != 0:
+        raise Exception("MDI Error: MDI_Get_python_plugin_mpi_world_ptr failed")
+    return python_plugin_mpi_world_ptr.value
+
 
 # MDI_Init
-mdi.MDI_Init.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.c_void_p]
-mdi.MDI_Init.restype = ctypes.c_int
-def MDI_Init(arg1, comm):
+mdi.MDI_Init_with_options.argtypes = [ctypes.POINTER(ctypes.c_char)]
+mdi.MDI_Init_with_options.restype = ctypes.c_int
+def MDI_Init(arg1):
     global world_comm
     global intra_code_comm
 
+    comm = None
+    if use_mpi4py:
+        comm = MPI.COMM_WORLD
+
+    # if this is a plugin code, get the plugin's MPI communicator
+    plugin_mode = MDI_Get_plugin_mode()
+    if ( plugin_mode == 1  and use_mpi4py ):
+        python_plugin_mpi_world_ptr = MDI_Get_python_plugin_mpi_world_ptr()
+        try:
+            # Assume python_plugin_mpi_world_ptr points to a pointer
+            __mdi_mpi_comm_ptr__ = ctypes.cast(python_plugin_mpi_world_ptr, ctypes.POINTER(ctypes.c_void_p))
+            ptr_type = ctypes.c_void_p
+            handle_t = ctypes.c_void_p
+            newobj = type(MPI.COMM_WORLD)()
+            handle_old = __mdi_mpi_comm_ptr__.contents
+            handle_new = handle_t.from_address(MPI._addressof(newobj))
+            handle_new.value = handle_old.value
+            __mdi_plugin_mpi_intra_comm__ = newobj
+
+            # Confirm that the new MPI communicator works
+            world_size = __mdi_plugin_mpi_intra_comm__.Get_size()
+        except:
+            # Assume python_plugin_mpi_world_ptr points to an int
+            __mdi_mpi_comm_ptr__ = ctypes.cast(python_plugin_mpi_world_ptr, ctypes.POINTER(ctypes.c_int))
+            handle_old_value = __mdi_mpi_comm_ptr__.contents.value
+            handle_t = ctypes.c_void_p
+            newobj = type(MPI.COMM_WORLD)()
+            handle_new = handle_t.from_address(MPI._addressof(newobj))
+            handle_new.value = handle_old_value
+            __mdi_plugin_mpi_intra_comm__ = newobj
+
+            # Confirm that the new MPI communicator works
+            world_size = __mdi_plugin_mpi_intra_comm__.Get_size()
+        comm = __mdi_plugin_mpi_intra_comm__
+
+
     # prepend the _language option, so that MDI knows this is a Python code
     arg1 = "_language Python " + arg1
 
@@ -431,26 +527,17 @@ def MDI_Init(arg1, comm):
         if not found_numpy:
             raise Exception("MDI Error: When using the MPI communication method, numpy must be available")
 
-        # check if MDI was responsible for initializing MPI
-        if comm is None:
-            comm = MPI.COMM_WORLD
+    if use_mpi4py:
+        world_comm = comm
+        intra_code_comm = comm
+        mpi_communicator = MPI._addressof(comm)
+        mpi_communicator_ptr = ctypes.c_void_p(mpi_communicator)
 
-    if comm is None:
-        mpi_communicator_ptr = None
-    else:
-        if use_mpi4py:
-            world_comm = comm
-            intra_code_comm = comm
-            mpi_communicator = MPI._addressof(comm)
-            mpi_communicator_ptr = ctypes.c_void_p(mpi_communicator)
-
-            # send basic information about the MPI communicator to the MDI libarary
-            mpi_rank = comm.Get_rank()
-            mpi_world_size = comm.Get_size()
-            mdi.MDI_Set_World_Rank(mpi_rank)
-            mdi.MDI_Set_World_Size(mpi_world_size)
-        else:
-            raise Exception("MDI Error: An MPI communicator was passed to MPI_Init, but MPI4Py is not found")
+        # send basic information about the MPI communicator to the MDI libarary
+        mpi_rank = comm.Get_rank()
+        mpi_world_size = comm.Get_size()
+        mdi.MDI_Set_World_Rank(mpi_rank)
+        mdi.MDI_Set_World_Size(mpi_world_size)
 
     # set the MPI4Py callback functions
     set_mpi4py_recv_callback()
@@ -463,25 +550,31 @@ def MDI_Init(arg1, comm):
 
     # call MDI_Init
     command = arg1.encode('utf-8')
-    ret = mdi.MDI_Init(ctypes.c_char_p(command), mpi_communicator_ptr )
+    ret = mdi.MDI_Init_with_options(ctypes.c_char_p(command) )
     if ret != 0:
         raise Exception("MDI Error: MDI_Init failed")
 
     return ret
 
-def MDI_Get_Intra_Code_MPI_Comm():
+def MDI_MPI_get_world_comm():
     global intra_code_comm
     return intra_code_comm
 
+def MDI_MPI_set_world_comm(new_comm):
+    global intra_code_comm
+    intra_code_comm = new_comm
+
 # MDI_Accept_Communicator
 mdi.MDI_Accept_Communicator.argtypes = [ctypes.POINTER(ctypes.c_int)]
 mdi.MDI_Accept_Communicator.restype = ctypes.c_int
-def MDI_Accept_Communicator():
+def MDI_Accept_communicator():
     comm = ctypes.c_int()
     ret = mdi.MDI_Accept_Communicator(ctypes.byref(comm))
     if ret != 0:
         raise Exception("MDI Error: MDI_Accept_Communicator failed")
     return comm.value
+def MDI_Accept_Communicator():
+    return MDI_Accept_communicator()
 
 # MDI_Send
 mdi.MDI_Send.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.c_int, ctypes.c_int, ctypes.c_int]
@@ -603,16 +696,18 @@ def MDI_Recv(arg2, arg3, arg4, buf = None):
 # MDI_Send_Command
 mdi.MDI_Send_Command.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.c_int]
 mdi.MDI_Send_Command.restype = ctypes.c_int
-def MDI_Send_Command(arg1, arg2):
+def MDI_Send_command(arg1, arg2):
     command = arg1.encode('utf-8')
     ret = mdi.MDI_Send_Command(ctypes.c_char_p(command), arg2)
     if ret != 0:
         raise Exception("MDI Error: MDI_Send_Command failed")
+def MDI_Send_Command(arg1, arg2):
+    return MDI_Send_command(arg1, arg2)
 
 # MDI_Recv_Command
 mdi.MDI_Recv_Command.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.c_int]
 mdi.MDI_Recv_Command.restype = ctypes.c_int
-def MDI_Recv_Command(arg2): 
+def MDI_Recv_command(arg2): 
     arg_size = ctypes.sizeof(ctypes.c_char)
     arg1 = (ctypes.c_char*(MDI_COMMAND_LENGTH*arg_size))()
 
@@ -629,11 +724,13 @@ def MDI_Recv_Command(arg2):
         delete_code_state(arg2)
 
     return presult
+def MDI_Recv_Command(arg2):
+    return MDI_Recv_command(arg2)
 
 # MDI_Conversion_Factor
 mdi.MDI_Conversion_Factor.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.POINTER(ctypes.c_char), ctypes.POINTER(ctypes.c_double)]
 mdi.MDI_Conversion_Factor.restype = ctypes.c_int
-def MDI_Conversion_Factor(arg1, arg2):
+def MDI_Conversion_factor(arg1, arg2):
     in_unit = arg1.encode('utf-8')
     out_unit = arg2.encode('utf-8')
     conversion = ctypes.c_double()
@@ -641,16 +738,20 @@ def MDI_Conversion_Factor(arg1, arg2):
     if ret != 0:
         raise Exception("MDI Error: MDI_Conversion_Factor failed")
     return conversion.value
+def MDI_Conversion_Factor(arg1, arg2):
+    return MDI_Conversion_factor(arg1, arg2)
 
 # MDI_Get_Role
 mdi.MDI_Get_Role.argtypes = [ctypes.POINTER(ctypes.c_int)]
 mdi.MDI_Get_Role.restype = ctypes.c_int
-def MDI_Get_Role():
+def MDI_Get_role():
     role = ctypes.c_int()
     ret = mdi.MDI_Get_Role(ctypes.byref(role))
     if ret != 0:
         raise Exception("MDI Error: MDI_Get_Role failed")
     return role.value
+def MDI_Get_Role():
+    return MDI_Get_role()
 
 
 #####################################
@@ -679,7 +780,7 @@ execute_command_func_type = ctypes.CFUNCTYPE(ctypes.c_int, ctypes.POINTER(ctypes
 MDI_Execute_Command_c = execute_command_func_type( MDI_Execute_Command_py )
 mdi.MDI_Set_Execute_Command_Func.argtypes = [execute_command_func_type, ctypes.c_void_p]
 mdi.MDI_Set_Execute_Command_Func.restype = ctypes.c_int
-def MDI_Set_Execute_Command_Func(func, class_obj):
+def MDI_Set_execute_command_func(func, class_obj):
     global execute_command_dict
 
     current_code = MDI_Get_Current_Code()
@@ -693,6 +794,8 @@ def MDI_Set_Execute_Command_Func(func, class_obj):
     ret = mdi.MDI_Set_Execute_Command_Func( MDI_Execute_Command_c, class_obj_pointer )
     if ret != 0:
         raise Exception("MDI Error: MDI_Set_Execute_Command_Func failed")
+def MDI_Set_Execute_Command_Func(func, class_obj):
+    return MDI_Set_execute_command_func(func, class_obj)
 
 
 
@@ -702,18 +805,20 @@ def MDI_Set_Execute_Command_Func(func, class_obj):
 # MDI_Register_Node
 mdi.MDI_Register_Node.argtypes = [ctypes.POINTER(ctypes.c_char)]
 mdi.MDI_Register_Node.restype = ctypes.c_int
-def MDI_Register_Node(arg1):
+def MDI_Register_node(arg1):
     node = arg1.encode('utf-8')
     ret = mdi.MDI_Register_Node(ctypes.c_char_p(node))
     if ret != 0:
         raise Exception("MDI Error: MDI_Register_Node failed")
 
     return ret
+def MDI_Register_Node(arg1):
+    return MDI_Register_node(arg1)
 
 # MDI_Check_Node_Exists
 mdi.MDI_Check_Node_Exists.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.c_int, ctypes.POINTER(ctypes.c_int)]
 mdi.MDI_Check_Node_Exists.restype = ctypes.c_int
-def MDI_Check_Node_Exists(arg1, arg2):
+def MDI_Check_node_exists(arg1, arg2):
     node = arg1.encode('utf-8')
 
     arg_size = ctypes.sizeof(ctypes.c_int)
@@ -725,11 +830,13 @@ def MDI_Check_Node_Exists(arg1, arg2):
     flag_cast = ctypes.cast(flag, ctypes.POINTER(ctypes.c_int)).contents
 
     return flag_cast.value
+def MDI_Check_Node_Exists(arg1, arg2):
+    return MDI_Check_node_exists(arg1, arg2)
 
 # MDI_Get_NNodes
 mdi.MDI_Get_NNodes.argtypes = [ctypes.c_int, ctypes.POINTER(ctypes.c_int)]
 mdi.MDI_Get_NNodes.restype = ctypes.c_int
-def MDI_Get_NNodes(arg2):
+def MDI_Get_nnodes(arg2):
     arg_size = ctypes.sizeof(ctypes.c_int)
     nnodes = (ctypes.c_int*arg_size)()
 
@@ -739,11 +846,13 @@ def MDI_Get_NNodes(arg2):
     nnodes_cast = ctypes.cast(nnodes, ctypes.POINTER(ctypes.c_int)).contents
 
     return nnodes_cast.value
+def MDI_Get_NNodes(arg2):
+    return MDI_Get_nnodes(arg2)
 
 # MDI_Get_Node
 mdi.MDI_Get_Node.argtypes = [ctypes.c_int, ctypes.c_int, ctypes.POINTER(ctypes.c_char)]
 mdi.MDI_Get_Node.restype = ctypes.c_int
-def MDI_Get_Node(index, arg2): 
+def MDI_Get_node(index, arg2): 
     arg_size = ctypes.sizeof(ctypes.c_char)
     node_name = (ctypes.c_char*(MDI_COMMAND_LENGTH*arg_size))()
 
@@ -755,11 +864,13 @@ def MDI_Get_Node(index, arg2):
     presult = ctypes.cast(result, ctypes.c_char_p).value
     presult = presult.decode('utf-8')
     return presult
+def MDI_Get_Node(index, arg2):
+    return MDI_Get_node(index, arg2)
 
 # MDI_Register_Command
 mdi.MDI_Register_Command.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.POINTER(ctypes.c_char)]
 mdi.MDI_Register_Command.restype = ctypes.c_int
-def MDI_Register_Command(arg1, arg2):
+def MDI_Register_command(arg1, arg2):
     node = arg1.encode('utf-8')
     command = arg2.encode('utf-8')
     ret = mdi.MDI_Register_Command(ctypes.c_char_p(node), ctypes.c_char_p(command))
@@ -767,11 +878,13 @@ def MDI_Register_Command(arg1, arg2):
         raise Exception("MDI Error: MDI_Get_Callback failed")
 
     return ret
+def MDI_Register_Command(arg1, arg2):
+    return MDI_Register_command(arg1, arg2)
 
 # MDI_Check_Command_Exists
 mdi.MDI_Check_Command_Exists.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.POINTER(ctypes.c_char), ctypes.c_int, ctypes.POINTER(ctypes.c_int)]
 mdi.MDI_Check_Command_Exists.restype = ctypes.c_int
-def MDI_Check_Command_Exists(arg1, command_name, arg2):
+def MDI_Check_command_exists(arg1, command_name, arg2):
     node = arg1.encode('utf-8')
     command = command_name.encode('utf-8')
 
@@ -784,11 +897,13 @@ def MDI_Check_Command_Exists(arg1, command_name, arg2):
     flag_cast = ctypes.cast(flag, ctypes.POINTER(ctypes.c_int)).contents
 
     return flag_cast.value
+def MDI_Check_Command_Exists(arg1, command_name, arg2):
+    return MDI_Check_command_exists(arg1, command_name, arg2)
 
 # MDI_Get_NCommands
 mdi.MDI_Get_NCommands.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.c_int, ctypes.POINTER(ctypes.c_int)]
 mdi.MDI_Get_NCommands.restype = ctypes.c_int
-def MDI_Get_NCommands(arg1, arg2):
+def MDI_Get_ncommands(arg1, arg2):
     node = arg1.encode('utf-8')
 
     arg_size = ctypes.sizeof(ctypes.c_int)
@@ -800,11 +915,13 @@ def MDI_Get_NCommands(arg1, arg2):
     ncommands_cast = ctypes.cast(ncommands, ctypes.POINTER(ctypes.c_int)).contents
 
     return ncommands_cast.value
+def MDI_Get_NCommands(arg1, arg2):
+    return MDI_Get_ncommands(arg1, arg2)
 
 # MDI_Get_Command
 mdi.MDI_Get_Command.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.c_int, ctypes.c_int, ctypes.POINTER(ctypes.c_char)]
 mdi.MDI_Get_Command.restype = ctypes.c_int
-def MDI_Get_Command(node_name, index, arg2): 
+def MDI_Get_command(node_name, index, arg2): 
     node = node_name.encode('utf-8')
 
     arg_size = ctypes.sizeof(ctypes.c_char)
@@ -815,11 +932,13 @@ def MDI_Get_Command(node_name, index, arg2):
         raise Exception("MDI Error: MDI_Get_Command failed")
 
     return c_ptr_to_py_str(command_name, MDI_COMMAND_LENGTH)
+def MDI_Get_Command(node_name, index, arg2):
+    return MDI_Get_command(node_name, index, arg2)
 
 # MDI_Register_Callback
 mdi.MDI_Register_Callback.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.POINTER(ctypes.c_char)]
 mdi.MDI_Register_Callback.restype = ctypes.c_int
-def MDI_Register_Callback(arg1, arg2):
+def MDI_Register_callback(arg1, arg2):
     node = arg1.encode('utf-8')
     callback = arg2.encode('utf-8')
     ret =  mdi.MDI_Register_Callback(ctypes.c_char_p(node), ctypes.c_char_p(callback))
@@ -827,11 +946,13 @@ def MDI_Register_Callback(arg1, arg2):
         raise Exception("MDI Error: MDI_Register_Callback failed")
 
     return ret
+def MDI_Register_Callback(arg1, arg2):
+    return MDI_Register_callback(arg1, arg2)
 
 # MDI_Check_Callback_Exists
 mdi.MDI_Check_Callback_Exists.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.POINTER(ctypes.c_char), ctypes.c_int, ctypes.POINTER(ctypes.c_int)]
 mdi.MDI_Check_Callback_Exists.restype = ctypes.c_int
-def MDI_Check_Callback_Exists(arg1, callback_name, arg2):
+def MDI_Check_callback_exists(arg1, callback_name, arg2):
     node = arg1.encode('utf-8')
     callback = callback_name.encode('utf-8')
 
@@ -844,11 +965,13 @@ def MDI_Check_Callback_Exists(arg1, callback_name, arg2):
     flag_cast = ctypes.cast(flag, ctypes.POINTER(ctypes.c_int)).contents
 
     return flag_cast.value
+def MDI_Check_Callback_Exists(arg1, callback_name, arg2):
+    return MDI_Check_callback_exists(arg1, callback_name, arg2)
 
 # MDI_Get_NCallbacks
 mdi.MDI_Get_NCallbacks.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.c_int, ctypes.POINTER(ctypes.c_int)]
 mdi.MDI_Get_NCallbacks.restype = ctypes.c_int
-def MDI_Get_NCallbacks(arg1, arg2):
+def MDI_Get_ncallbacks(arg1, arg2):
     node = arg1.encode('utf-8')
 
     arg_size = ctypes.sizeof(ctypes.c_int)
@@ -860,11 +983,13 @@ def MDI_Get_NCallbacks(arg1, arg2):
     ncallbacks_cast = ctypes.cast(ncallbacks, ctypes.POINTER(ctypes.c_int)).contents
 
     return ncallbacks_cast.value
+def MDI_Get_NCallbacks(arg1, arg2):
+    return MDI_Get_ncallbacks(arg1, arg2)
 
 # MDI_Get_Callback
 mdi.MDI_Get_Callback.argtypes = [ctypes.POINTER(ctypes.c_char), ctypes.c_int, ctypes.c_int, ctypes.POINTER(ctypes.c_char)]
 mdi.MDI_Get_Callback.restype = ctypes.c_int
-def MDI_Get_Callback(node_name, index, arg2): 
+def MDI_Get_callback(node_name, index, arg2): 
     node = node_name.encode('utf-8')
 
     arg_size = ctypes.sizeof(ctypes.c_char)
@@ -875,3 +1000,31 @@ def MDI_Get_Callback(node_name, index, arg2):
         raise Exception("MDI Error: MDI_Get_Callback failed")
 
     return c_ptr_to_py_str(callback_name, MDI_COMMAND_LENGTH)
+def MDI_Get_Callback(node_name, index, arg2):
+    return MDI_Get_callback(node_name, index, arg2)
+
+
+
+##################################################
+# Plugin functions                               #
+##################################################
+
+# MDI_Plugin_get_argc
+mdi.MDI_Plugin_get_argc.argtypes = [ctypes.POINTER(ctypes.c_int)]
+mdi.MDI_Plugin_get_argc.restype = ctypes.c_int
+def MDI_Plugin_get_argc():
+    argc = ctypes.c_int()
+    ret = mdi.MDI_Plugin_get_argc(ctypes.byref(argc))
+    if ret != 0:
+        raise Exception("MDI Error: MDI_Plugin_get_argc failed")
+    return argc.value
+
+# MDI_Plugin_get_arg
+mdi.MDI_Plugin_get_arg.argtypes = [ctypes.c_int, ctypes.POINTER(ctypes.c_char_p)]
+mdi.MDI_Plugin_get_arg.restype = ctypes.c_int
+def MDI_Plugin_get_arg(index):
+    arg = ctypes.c_char_p()
+    ret = mdi.MDI_Plugin_get_arg(index, ctypes.byref(arg))
+    if ret != 0:
+        raise Exception("MDI Error: MDI_Plugin_get_arg failed")
+    return c_ptr_to_py_str(arg, len(arg.value) )
