@@ -44,14 +44,12 @@ int general_init(const char* options) {
   char* hostname;
   int port;
   char* output_file;
-  char* driver_name;
   char* language_argument = ((char*)"");
   int has_role = 0;
   int has_method = 0;
   int has_name = 0;
   int has_hostname = 0;
   int has_port = 0;
-  int has_driver_name = 0;
   int has_plugin_path = 0;
   int has_output_file = 0;
 
@@ -148,16 +146,6 @@ int general_init(const char* options) {
       output_file = argv[iarg+1];
       iarg += 2;
     }
-    //-driver_name
-    else if (strcmp(argv[iarg],"-driver_name") == 0) {
-      if (iarg+2 > argc) {
-	mdi_error("Error in MDI_Init: Argument missing from -driver_name option");
-	return 1;
-      }
-      driver_name = argv[iarg+1];
-      has_driver_name = 1;
-      iarg += 2;
-    }
     //-plugin_path
     else if (strcmp(argv[iarg],"-plugin_path") == 0) {
       if (iarg+2 > argc) {
@@ -253,7 +241,6 @@ int general_init(const char* options) {
 
   // get the appropriate MPI communicator to use
   MPI_Comm mpi_communicator;
-  int mpi_rank = 0;
   ret = MPI_Initialized(&mpi_init_flag);
   if ( ret != 0 ) {
     mdi_error("Error in MDI_Init: MPI_Initialized failed");
@@ -261,16 +248,16 @@ int general_init(const char* options) {
   }
   if ( mpi_init_flag == 0 ) {
     mpi_communicator = 0;
-    mpi_rank = 0;
   }
   else {
     if ( this_code->language == MDI_LANGUAGE_PYTHON ) {
       mpi_communicator = 0;
-      mpi_rank = world_rank;
     }
     else {
       mpi_communicator = MPI_COMM_WORLD;
-      MPI_Comm_rank(mpi_communicator, &mpi_rank);
+      MPI_Comm_rank(mpi_communicator, &world_rank);
+      MPI_Comm_size(mpi_communicator, &world_size);
+      this_code->intra_rank = world_rank;
     }
   }
 
@@ -339,6 +326,10 @@ int general_init(const char* options) {
       if ( this_code->intra_rank == 0 ) {
 	tcp_listen(port);
       }
+      else {
+	// If this isn't rank 0, just set tcp_socket to > 0 so that accept_communicator knows we are using TCP
+	tcp_socket = 1;
+      }
     }
     else if ( strcmp(method, "LINK") == 0 ) {
       //library_initialize();
@@ -372,12 +363,14 @@ int general_init(const char* options) {
       if ( this_code->intra_rank == 0 ) {
 	tcp_request_connection(port, hostname);
       }
+      else {
+	// If this isn't rank 0, just set tcp_socket to > 0 so that accept_communicator knows we are using TCP
+	if ( this_code->intra_rank != 0 ) {
+	  tcp_socket = 1;
+	}
+      }
     }
     else if ( strcmp(method, "LINK") == 0 ) {
-      if ( has_driver_name == 0 ) {
-	mdi_error("Error in MDI_Init: -driver_name option not provided");
-	return 1;
-      }
       library_initialize();
     }
     else if ( strcmp(method, "TEST") == 0 ) {
@@ -409,20 +402,21 @@ int general_init(const char* options) {
  *
  */
 int general_accept_communicator() {
-  // give the library method an opportunity to update the current code
+  // Give the library method an opportunity to update the current code
   library_accept_communicator();
 
-  // if MDI hasn't returned some connections, do that now
+  // If MDI hasn't returned some connections, do that now
   code* this_code = get_code(current_code);
   if ( this_code->returned_comms < this_code->next_comm - 1 ) {
     this_code->returned_comms++;
     return this_code->returned_comms;
   }
 
-  // check for any production codes connecting via TCP
+  // Check for any production codes connecting via TCP
   if ( tcp_socket > 0 ) {
 
-    //accept a connection via TCP
+    // Accept a connection via TCP
+    // NOTE: If this is not intra_rank==0, this will always create a dummy communicator
     tcp_accept_connection();
 
     // if MDI hasn't returned some connections, do that now
