@@ -34,7 +34,7 @@
 // MDI version numbers
 #define MDI_MAJOR_VERSION_ 1
 #define MDI_MINOR_VERSION_ 2
-#define MDI_PATCH_VERSION_ 6
+#define MDI_PATCH_VERSION_ 10
 
 // length of an MDI command in characters
 #define MDI_COMMAND_LENGTH_ 12
@@ -182,6 +182,18 @@ extern MPI_Comm mdi_mpi_comm_world;
  * Only used for Python plugins */
 extern void* python_plugin_mpi_world_ptr;
 
+/*! \brief Unedited command-line options for currently running plugin */
+extern char* plugin_unedited_options;
+
+/*! \brief Command-line options for currently running plugin */
+extern char* plugin_options;
+
+/*! \brief Argument count for plugin command-line options */
+extern int plugin_argc;
+
+/*! \brief Argument vector for plugin command-line options */
+extern char** plugin_argv;
+
 /*! \brief Python callback pointer for MPI_Recv */
 extern int (*mpi4py_recv_callback)(void*, int, int, int, MDI_Comm_Type);
 
@@ -203,6 +215,12 @@ extern int (*mpi4py_size_callback)(int);
 /*! \brief Python callback pointer for MPI_Comm_barrier */
 extern int (*mpi4py_barrier_callback)(int);
 
+/*! \brief Size of MPI_COMM_WORLD */
+extern int world_size;
+
+/*! \brief Rank of this process within MPI_COMM_WORLD */
+extern int world_rank;
+
 int vector_init(vector* v, size_t stride);
 int vector_push_back(vector* v, void* element);
 void* vector_get(vector* v, int index);
